from rich import print
from zona.models import Item, ItemType
from zona.metadata import parse_metadata
from zona import markdown as zmd
from zona.templates import Templater
from zona.layout import Layout, discover_layout
from zona.config import ZonaConfig
from zona import util
from pathlib import Path


class ZonaBuilder:
    def __init__(
        self, cli_root: Path | None = None, cli_output: Path | None = None
    ):
        self.layout: Layout = discover_layout(cli_root, cli_output)
        self.config: ZonaConfig = ZonaConfig.from_file(
            self.layout.root / "config.yml"
        )
        self.items: list[Item] = []
        self.item_map: dict[Path, Item] = {}

    def _discover(self):
        layout = self.layout
        items: list[Item] = []

        base = layout.root / layout.content
        for path in base.rglob("*"):
            if path.is_file() and not util.should_ignore(
                path, patterns=self.config.ignore, base=base
            ):
                # we only parse markdown files not in static/
                destination = layout.output / path.relative_to(base)
                item = Item(
                    source=path,
                    destination=destination,
                    url=str(destination.relative_to(layout.output)),
                )
                if path.name.endswith(".md") and not path.is_relative_to(
                    layout.root / "content" / "static"
                ):
                    item.metadata, item.content = parse_metadata(path)
                    item.type = ItemType.MARKDOWN
                    item.copy = False
                    name = destination.stem
                    if name == "index":
                        item.destination = item.destination.with_suffix(".html")
                    else:
                        relative = path.relative_to(base).with_suffix("")
                        name = relative.stem
                        item.destination = (
                            layout.output
                            / relative.parent
                            / name
                            / "index.html"
                        )
                    rel_url = item.destination.parent.relative_to(layout.output)
                    item.url = (
                        "" if rel_url == Path(".") else rel_url.as_posix()
                    )
                items.append(item)
        self.items = items

    def _build(self):
        assert self.items
        templater = Templater(self.layout.templates)
        self.item_map = {item.source.resolve(): item for item in self.items}
        # print(item_map)
        for item in self.item_map.values():
            dst = item.destination
            # print(item)
            # create parent dirs if needed
            if item.type == ItemType.MARKDOWN:
                assert item.content is not None
                # parse markdown and render as html
                raw_html = zmd.md_to_html(
                    content=item.content,
                    resolve_links=True,
                    source=item.source,
                    layout=self.layout,
                    item_map=self.item_map,
                )
                # TODO: test this
                rendered = templater.render_item(item, raw_html)
                util.ensure_parents(dst)
                dst.write_text(rendered, encoding="utf-8")
            else:
                if item.copy:
                    util.copy_static_file(item.source, dst)

    def build(self):
        self._discover()
        self._build()
